@@ -1,12 +1,9 @@
 #!/usr/bin/python3.8
+from typing import Optional
+
 import requests
 from bs4 import BeautifulSoup
-
-header = "\033[4m"
-red = "\033[31m"
-green = "\033[32m"
-blue = "\033[34m"
-normal = "\033[0m"
+from BibleHubScrapper import config
 
 
 class BibleHubQuery:
@@ -14,55 +11,53 @@ class BibleHubQuery:
     Can optionally contain Treasury of Scriptures(tsks), cross references(crfs) and lexicons
     Use the \"query\" function to create these objects. """
 
-    def __init__(self, passage, text, version, crfs=None, tsks=None, lexicons=None):
+    def __init__(self, passage: str, text: str, version: str, crfs: list = None, tsks: list = None,
+                 lexicons: list = None):
         if lexicons is None:
             lexicons = []
         if tsks is None:
             tsks = []
         if crfs is None:
             crfs = []
-        assert isinstance(text, str)
-        assert isinstance(crfs, list)
-        assert isinstance(tsks, list)
-        assert isinstance(lexicons, list)
-        assert isinstance(version, str)
         self.passage = passage
         self.text = text
         self.version = version
         self.crfs = crfs
         self.tsks = tsks
         self.lexicons = lexicons
+        self._init_info()
 
-    def __str__(self) -> str:
+    def _init_info(self) -> None:
         response = self.format_verse()
         if self.crfs:
-            response += "\n\n{header}Cross References{normal}\n{crfs}".format(header=header, normal=normal,
-                                                                              crfs=self.format_crfs())
+            response += "{header}{crfs}".format(header=config.format_header("Cross References"),
+                                                crfs=self.format_crfs())
         if self.tsks:
-            response += "\n\n{header}Treasury Of Scripture{normal}\n{tsks}".format(header=header, normal=normal,
-                                                                                   tsks=self.format_tsks())
+            response += "{header}{tsks}".format(header=config.format_header("Treasury Of Scripture"),
+                                                tsks=self.format_tsks())
         if self.lexicons:
-            response += "\n\n{header}Lexicon{normal}\n{lexicon}".format(header=header, normal=normal,
-                                                                        lexicon=self.format_lexicons())
-        return response
+            response += "{header}{lexicon}".format(header=config.format_header("Lexicon"),
+                                                   lexicon=self.format_lexicons())
+        self.info = response
+
+    def __str__(self) -> str:
+        return self.info
 
     def format_crfs(self) -> str:
         response = ''
         for crf in self.crfs:
-            response += crf + '\n'
+            response += config.format_verse(crf['reference'], crf['text'])
         return response
 
     def format_tsks(self) -> str:
         response = []
         for tsk in self.tsks:
             response.append(tsk)
-        return '\n'.join(response)
+        return '\n'.join(response) + '\n'
 
     def format_verse(self) -> str:
-        return "{blue}{reference} ({version}) - {normal}{verse}".format(
-            blue=blue, reference=self.passage, version=self.version.upper(),
-            normal=normal, verse=self.text
-        )
+        return config.format_verse('{reference} ({version})'.format(reference=self.passage, version=self.version),
+                                   self.text)
 
     def format_lexicons(self) -> str:
         response = []
@@ -111,11 +106,23 @@ def _format_query(text) -> str:
 
 
 def _get_crfs(whole) -> list:
-    crfs_list = whole.find("div", {"id": "crf"}).find_all("span", {"class": "crossverse"})
+    response = []
+    crf_section = whole.find("div", {"id": "crf"})
+    crfs_list = crf_section.find_all("span", {"class": "crossverse"})
+    text = crf_section.get_text()
+    assert isinstance(text, str)
     crfs = []
     for crf in crfs_list:
         crfs.append(crf.get_text())
-    return crfs
+    # May Create edge cases, look for a better way sometime
+    for i in range(len(crfs)):
+        begin = text.find(crfs[i]) + len(crfs[i])
+        if i < len(crfs) - 1:
+            end = text.find(crfs[i + 1])
+        else:
+            end = len(text)
+        response.append({"reference": crfs[i], "text": text[begin:end]})
+    return response
 
 
 def _get_lexicon(page) -> list:
@@ -141,17 +148,19 @@ def _get_tsks(page) -> list:
     return tsks
 
 
-def _find_version(whole, ver) -> BeautifulSoup:
+def _find_version(whole, ver) -> Optional[BeautifulSoup]:
     versions = whole.find_all_next("span", "versiontext")
     for version in versions:
         if version.a.attrs['href'].startswith(ver):
             return version
-    return versions[0]
+    return None
 
 
-def _get_passage(whole, version) -> str:
+def _get_passage(whole, version) -> Optional[str]:
     version_formatted = "/%s/" % version.lower()
     first = _find_version(whole, version_formatted)
+    if first is None:
+        return None
     second = first.find_next_sibling("span", {"class": "versiontext"})
     beg = first.get_text()
     block = whole.get_text()
@@ -163,7 +172,7 @@ def _get_passage(whole, version) -> str:
     return verse
 
 
-def query(reference, version="niv", get_tsks=True, get_crfs=True, get_lexicons=True) -> BibleHubQuery:
+def query(reference: str, version="niv", get_tsks=True, get_crfs=True, get_lexicons=True) -> BibleHubQuery:
     """Returns up to **one** verse along with the information provided by Biblehub
 
     :param reference: The reference to be parsed (i.e Genesis 1:1)
@@ -179,6 +188,9 @@ def query(reference, version="niv", get_tsks=True, get_crfs=True, get_lexicons=T
     page = _query_site(url)
     whole = page.find("div", {"id": "par"})
     verse = _get_passage(whole, version)
+    if verse is None:
+        version = "niv"
+        verse = _get_passage(whole, version)
     lexicons = None
     if get_lexicons:
         lexicons = _get_lexicon(page)
@@ -188,5 +200,5 @@ def query(reference, version="niv", get_tsks=True, get_crfs=True, get_lexicons=T
     crfs = None
     if get_crfs:
         crfs = _get_crfs(page)
-    hub_query = BibleHubQuery(reference.title(), verse, version, crfs, tsks, lexicons)
+    hub_query = BibleHubQuery(reference.title(), verse, version.upper(), crfs, tsks, lexicons)
     return hub_query
